{% extends "admin/change_form.html" %}
{% load i18n l10n admin_urls %}

{% block breadcrumbs %}
<div class="breadcrumbs">
<a href="{% url 'admin:index' %}">{% trans 'Home' %}</a>
&rsaquo; <a href="{% url 'admin:app_list' app_label=opts.app_label %}">{{ opts.app_config.verbose_name }}</a>
&rsaquo; <a href="{% url opts|admin_urlname:'changelist' %}">{{ opts.verbose_name_plural|capfirst|escape }}</a>
&rsaquo; {{ title }}
</div>
{% endblock %}

{% block submit_buttons_top %}{% endblock %}
{% block submit_buttons_bottom %}{% endblock %}

{% block object-tools %}{% endblock %}

{% block field_sets %}
    {% for obj in queryset %}
    <input type="hidden" name="{{ action_checkbox_name }}" value="{{ obj.pk|unlocalize }}" />
    {% endfor %}
    {% for name, value in hidden_fields %}
    <input type="hidden" name="{{ name }}" value="{{ value }}" />
    {% endfor %}
    <input type="hidden" name="action" value="{{ action }}" />
    <table>
    {{ form.as_table }}
    </table>
    <input type="submit" value="{% trans "Continue" %}" />
    <a href="#" onclick="window.history.back(); return false;" class="button cancel-link">{% trans "Cancel" %}</a>
{% endblock %}
