# Generated by Django 3.2.25 on 2024-05-09 08:48

from django.db import migrations
import leprikon.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ("leprikon", "0083_statgroup"),
    ]

    operations = [
        migrations.AddField(
            model_name="subjectregistration",
            name="cached_balance",
            field=leprikon.models.fields.PriceField(
                decimal_places=0, default=0, editable=False, max_digits=10, verbose_name="payments balance"
            ),
            preserve_default=False,
        ),
    ]
