# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2022-02-20 19:52
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("leprikon", "0067_remove_rocketchat"),
    ]

    operations = [
        migrations.AddField(
            model_name="participant",
            name="answers",
            field=models.TextField(blank=True, default="{}", editable=False, verbose_name="additional answers"),
        ),
    ]
