{% extends "leprikon/default.html" %}
{% load i18n %}

{% block content %}

<h1>{% block title %}{% trans 'Email with instructions sent' %}{% endblock %}</h1>

<p>{% trans "We've emailed you instructions for setting your password. You should be receiving them shortly." %}</p>

<p>{% trans "If you don't receive an email, please make sure you've entered the address you registered with, and check your spam folder." %}</p>

{% endblock %}
