{% extends 'leprikon/default.html' %}
{% load i18n leprikon_tags sekizai_tags %}

{% block content %}

<h1>{% block title %}{{ event.name }}{% endblock %}</h1>

{% include 'leprikon/activity_preview.html' %}

{% endblock %}
