# -*- coding: utf-8 -*-
# Generated by Django 1.11.27 on 2020-02-16 22:13
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import leprikon.models.fields
import localflavor.generic.models


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0046_attachment_events'),
    ]

    operations = [
        migrations.AddField(
            model_name='leprikonsite',
            name='pr_print_setup',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='leprikon.PrintSetup', verbose_name='default payment request print setup'),
        ),
        migrations.AddField(
            model_name='subject',
            name='pr_print_setup',
            field=models.ForeignKey(blank=True, help_text='Only use to set value specific for this subject.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='leprikon.PrintSetup', verbose_name='payment request print setup'),
        ),
        migrations.AddField(
            model_name='subjecttype',
            name='pr_print_setup',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='leprikon.PrintSetup', verbose_name='payment request print setup'),
        ),
        migrations.AlterField(
            model_name='billinginfo',
            name='city',
            field=models.CharField(blank=True, default='', max_length=150, verbose_name='city'),
        ),
        migrations.AlterField(
            model_name='billinginfo',
            name='company_num',
            field=models.CharField(blank=True, default='', max_length=8, verbose_name='company number'),
        ),
        migrations.AlterField(
            model_name='billinginfo',
            name='contact_person',
            field=models.CharField(blank=True, default='', max_length=60, verbose_name='contact person'),
        ),
        migrations.AlterField(
            model_name='billinginfo',
            name='postal_code',
            field=leprikon.models.fields.PostalCodeField(blank=True, default='', verbose_name='postal code'),
        ),
        migrations.AlterField(
            model_name='billinginfo',
            name='street',
            field=models.CharField(blank=True, default='', max_length=150, verbose_name='street'),
        ),
        migrations.AlterField(
            model_name='billinginfo',
            name='vat_number',
            field=models.CharField(blank=True, default='', max_length=12, verbose_name='VAT number'),
        ),
        migrations.AlterField(
            model_name='organization',
            name='bic',
            field=localflavor.generic.models.BICField(blank=True, default='', max_length=11, verbose_name='BIC (SWIFT)'),
        ),
        migrations.AlterField(
            model_name='organization',
            name='city',
            field=models.CharField(blank=True, default='', max_length=150, verbose_name='city'),
        ),
        migrations.AlterField(
            model_name='organization',
            name='company_num',
            field=models.CharField(blank=True, default='', max_length=8, verbose_name='company number'),
        ),
        migrations.AlterField(
            model_name='organization',
            name='email',
            field=leprikon.models.fields.EmailField(blank=True, default='', max_length=254, verbose_name='email address'),
        ),
        migrations.AlterField(
            model_name='organization',
            name='iban',
            field=localflavor.generic.models.IBANField(blank=True, default='', include_countries=None, max_length=34, use_nordea_extensions=False, verbose_name='IBAN'),
        ),
        migrations.AlterField(
            model_name='organization',
            name='phone',
            field=models.CharField(blank=True, default='', max_length=30, verbose_name='phone'),
        ),
        migrations.AlterField(
            model_name='organization',
            name='postal_code',
            field=leprikon.models.fields.PostalCodeField(blank=True, default='', verbose_name='postal code'),
        ),
        migrations.AlterField(
            model_name='organization',
            name='street',
            field=models.CharField(blank=True, default='', max_length=150, verbose_name='street'),
        ),
        migrations.AlterField(
            model_name='organization',
            name='vat_number',
            field=models.CharField(blank=True, default='', max_length=12, verbose_name='VAT number'),
        ),
        migrations.AlterField(
            model_name='subjectregistrationbillinginfo',
            name='city',
            field=models.CharField(blank=True, default='', max_length=150, verbose_name='city'),
        ),
        migrations.AlterField(
            model_name='subjectregistrationbillinginfo',
            name='company_num',
            field=models.CharField(blank=True, default='', max_length=8, verbose_name='company number'),
        ),
        migrations.AlterField(
            model_name='subjectregistrationbillinginfo',
            name='contact_person',
            field=models.CharField(blank=True, default='', max_length=60, verbose_name='contact person'),
        ),
        migrations.AlterField(
            model_name='subjectregistrationbillinginfo',
            name='postal_code',
            field=leprikon.models.fields.PostalCodeField(blank=True, default='', verbose_name='postal code'),
        ),
        migrations.AlterField(
            model_name='subjectregistrationbillinginfo',
            name='street',
            field=models.CharField(blank=True, default='', max_length=150, verbose_name='street'),
        ),
        migrations.AlterField(
            model_name='subjectregistrationbillinginfo',
            name='vat_number',
            field=models.CharField(blank=True, default='', max_length=12, verbose_name='VAT number'),
        ),
    ]
