{% for group in groups %}

    {% if group.objects.count %}
        <h2 class="group_name">{{ group.group.plural }}</h2>

        {% for orderable in group.objects %}
            {% include 'leprikon/orderable_preview.html' %}
        {% endfor %}
    {% endif %}

{% endfor %}
