# Generated by Django 3.2.16 on 2023-01-20 20:11

from django.db import migrations
import djangocms_text_ckeditor.fields


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0076_journalentry_period'),
    ]

    operations = [
        migrations.AddField(
            model_name='place',
            name='description',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', verbose_name='description'),
        ),
    ]
