# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2020-03-21 12:57
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("leprikon", "0048_birth_date"),
    ]

    operations = [
        migrations.AddField(
            model_name="courselistplugin",
            name="target_groups",
            field=models.ManyToManyField(
                blank=True,
                help_text="Keep empty to skip searching by target groups.",
                related_name="+",
                to="leprikon.TargetGroup",
                verbose_name="target groups",
            ),
        ),
        migrations.AddField(
            model_name="eventlistplugin",
            name="target_groups",
            field=models.ManyToManyField(
                blank=True,
                help_text="Keep empty to skip searching by target groups.",
                related_name="+",
                to="leprikon.TargetGroup",
                verbose_name="target groups",
            ),
        ),
    ]
