# -*- coding: utf-8 -*-
# Generated by Django 1.11.27 on 2020-02-13 12:07
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0044_billinginfo_contacts'),
    ]

    operations = [
        migrations.AlterField(
            model_name='subjectregistration',
            name='slug',
            field=models.SlugField(editable=False, max_length=250, null=True),
        ),
    ]
