# Generated by Django 3.2.25 on 2024-10-08 19:47

from django.db import migrations
import multiselectfield.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0085_alter_subjectregistration_cached_balance'),
    ]

    operations = [
        migrations.AlterField(
            model_name='subjectattachment',
            name='events',
            field=multiselectfield.db.fields.MultiSelectField(blank=True, choices=[('registration_received', 'registration received'), ('registration_approved', 'registration approved'), ('registration_refused', 'registration refused'), ('registration_payment_request', 'payment requested'), ('registration_refund_offer', 'payment requested'), ('registration_canceled', 'registration canceled'), ('discount_granted', 'discount granted'), ('payment_received', 'payment received'), ('payment_returned', 'payment returned')], default=[], help_text='The attachment will be sent with notification on selected events.', max_length=250, verbose_name='send when'),
        ),
        migrations.AlterField(
            model_name='subjecttypeattachment',
            name='events',
            field=multiselectfield.db.fields.MultiSelectField(blank=True, choices=[('registration_received', 'registration received'), ('registration_approved', 'registration approved'), ('registration_refused', 'registration refused'), ('registration_payment_request', 'payment requested'), ('registration_refund_offer', 'payment requested'), ('registration_canceled', 'registration canceled'), ('discount_granted', 'discount granted'), ('payment_received', 'payment received'), ('payment_returned', 'payment returned')], default=[], help_text='The attachment will be sent with notification on selected events.', max_length=250, verbose_name='send when'),
        ),
    ]
