# Generated by Django 3.2.25 on 2025-07-17 15:51

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0089_orderables'),
    ]

    operations = [
        migrations.AlterField(
            model_name='registration',
            name='calendar_event',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='registration', to='leprikon.calendarevent', verbose_name='calendar event'),
        ),
    ]
