{% load sekizai_tags static %}

{% addtoblock "css" %}
<link rel="stylesheet" href="{% static 'leprikon/lib/bootstrap-multiselect.css' %}"/>
{% endaddtoblock %}

{% addtoblock "js" %}
<script src="{% static 'leprikon/lib/bootstrap-multiselect.min.js' %}"></script>
{% endaddtoblock %}

